@@ -1,6 +1,6 @@
 from typing import TYPE_CHECKING
 
-__version__ = "0.15.2"
+__version__ = "0.17.0"
 
 
 if TYPE_CHECKING:
@@ -33,8 +33,9 @@ def setup_sphinx(app: "Sphinx"):
 
     app.add_post_transform(MystReferenceResolver)
 
-    for name, default in MdParserConfig().as_dict().items():
-        if not name == "renderer":
+    for name, default, field in MdParserConfig().as_triple():
+        if not field.metadata.get("docutils_only", False):
+            # TODO add types?
             app.add_config_value(f"myst_{name}", default, "env")
 
     app.connect("builder-inited", create_myst_config)
@@ -53,8 +54,8 @@ def create_myst_config(app):
 
     values = {
         name: app.config[f"myst_{name}"]
-        for name in MdParserConfig().as_dict().keys()
-        if name != "renderer"
+        for name, _, field in MdParserConfig().as_triple()
+        if not field.metadata.get("docutils_only", False)
     }
 
     try:
