@@ -1,9 +1,13 @@
 """MyST specific directives"""
-from typing import List
+from copy import copy
+from typing import List, Tuple, cast
 
 from docutils import nodes
 from docutils.parsers.rst import directives
 from sphinx.directives import SphinxDirective
+from sphinx.util.docutils import SphinxRole
+
+from myst_parser.mocking import MockState
 
 
 def align(argument):
@@ -17,6 +21,19 @@ def figwidth_value(argument):
         return directives.length_or_percentage_or_unitless(argument, "px")
 
 
+class SubstitutionReferenceRole(SphinxRole):
+    """Implement substitution references as a role.
+
+    Note, in ``docutils/parsers/rst/roles.py`` this is left unimplemented.
+    """
+
+    def run(self) -> Tuple[List[nodes.Node], List[nodes.system_message]]:
+        subref_node = nodes.substitution_reference(self.rawtext, self.text)
+        self.set_source_info(subref_node, self.lineno)  # type: ignore[arg-type]
+        subref_node["refname"] = nodes.fully_normalize_name(self.text)
+        return [subref_node], []
+
+
 class FigureMarkdown(SphinxDirective):
     """Directive for creating a figure with Markdown compatible syntax.
 
@@ -47,18 +64,20 @@ class FigureMarkdown(SphinxDirective):
         figclasses = self.options.pop("class", None)
         align = self.options.pop("align", None)
 
-        # parser = default_parser(self.env.myst_config)
+        if not isinstance(self.state, MockState):
+            return [self.figure_error("Directive is only supported in myst parser")]
+        state = cast(MockState, self.state)
 
-        node = nodes.Element()
-        # TODO test that we are using myst parser
         # ensure html image enabled
-        myst_extensions = self.state._renderer.config.get("myst_extensions", set())
+        myst_extensions = copy(state._renderer.md_config.enable_extensions)
+        node = nodes.Element()
         try:
-            self.state._renderer.config.setdefault("myst_extensions", set())
-            self.state._renderer.config["myst_extensions"].add("html_image")
-            self.state.nested_parse(self.content, self.content_offset, node)
+            state._renderer.md_config.enable_extensions = list(
+                state._renderer.md_config.enable_extensions
+            ) + ["html_image"]
+            state.nested_parse(self.content, self.content_offset, node)
         finally:
-            self.state._renderer.config["myst_extensions"] = myst_extensions
+            state._renderer.md_config.enable_extensions = myst_extensions
 
         if not len(node.children) == 2:
             return [
