@@ -5,7 +5,7 @@ import os
 import re
 import sys
 from pathlib import Path
-from typing import TYPE_CHECKING, List, Optional, Tuple, Type
+from typing import TYPE_CHECKING, Any, List, Optional, Tuple, Type
 
 from docutils import nodes
 from docutils.parsers.rst import Directive, DirectiveError
@@ -31,28 +31,50 @@ class MockInliner:
     This is parsed to role functions.
     """
 
-    def __init__(self, renderer: "DocutilsRenderer", lineno: int):
+    def __init__(self, renderer: "DocutilsRenderer"):
+        """Initialize the mock inliner."""
         self._renderer = renderer
+        # here we mock that the `parse` method has already been called
+        # which is where these attributes are set (via the RST state Memo)
         self.document = renderer.document
         self.reporter = renderer.document.reporter
-        if not hasattr(self.reporter, "get_source_and_line"):
-            # TODO this is called by some roles,
-            # but I can't see how that would work in RST?
-            self.reporter.get_source_and_line = lambda l: (self.document["source"], l)
-        self.parent = renderer.current_node
         self.language = renderer.language_module_rst
+        self.parent = renderer.current_node
+
+        if not hasattr(self.reporter, "get_source_and_line"):
+            # In docutils this is set by `RSTState.runtime_init`
+            self.reporter.get_source_and_line = lambda l: (self.document["source"], l)
+
         self.rfc_url = "rfc%d.html"
 
     def problematic(
         self, text: str, rawsource: str, message: nodes.system_message
     ) -> nodes.problematic:
+        """Record a system message from parsing."""
         msgid = self.document.set_id(message, self.parent)
-        problematic = nodes.problematic(rawsource, rawsource, refid=msgid)
+        problematic = nodes.problematic(rawsource, text, refid=msgid)
         prbid = self.document.set_id(problematic)
         message.add_backref(prbid)
         return problematic
 
-    # TODO add parse method
+    def parse(
+        self, text: str, lineno: int, memo: Any, parent: nodes.Node
+    ) -> Tuple[List[nodes.Node], List[nodes.system_message]]:
+        """Parse the text and return a list of nodes."""
+        # note the only place this is normally called,
+        # is by `RSTState.inline_text`, or in directives: `self.state.inline_text`,
+        # and there the state parses its own parent
+        # self.reporter = memo.reporter
+        # self.document = memo.document
+        # self.language = memo.language
+        with self._renderer.current_node_context(parent):
+            # the parent is never actually appended to though,
+            # so we make a temporary parent to parse into
+            container = nodes.Element()
+            with self._renderer.current_node_context(container):
+                self._renderer.nested_render_text(text, lineno, inline=True)
+
+        return container.children, []
 
     def __getattr__(self, name: str):
         """This method is only be called if the attribute requested has not
@@ -87,6 +109,7 @@ class MockState:
         self.document = renderer.document
         self.reporter = renderer.document.reporter
         self.state_machine = state_machine
+        self.inliner = MockInliner(renderer)
 
         class Struct:
             document = self.document
@@ -95,7 +118,7 @@ class MockState:
             title_styles: List[str] = []
             section_level = max(renderer._level_to_elem)
             section_bubble_up_kludge = False
-            inliner = MockInliner(renderer, lineno)
+            inliner = self.inliner
 
         self.memo = Struct
 
@@ -113,14 +136,14 @@ class MockState:
         if option_presets:
             raise MockingError("parse_directive_block: option_presets not implemented")
         # TODO should argument_str always be ""?
-        arguments, options, body_lines = parse_directive_text(
+        arguments, options, body_lines, content_offset = parse_directive_text(
             directive, "", "\n".join(content)
         )
         return (
             arguments,
             options,
             StringList(body_lines, source=content.source),
-            line_offset + len(content) - len(body_lines),
+            line_offset + content_offset,
         )
 
     def nested_parse(
@@ -132,19 +155,24 @@ class MockState:
         state_machine_class=None,
         state_machine_kwargs=None,
     ) -> None:
-        """Perform a nested parse of the input block, with ``node`` as the parent."""
-        sm_match_titles = self.state_machine.match_titles
-        render_match_titles = self._renderer.md_env.get("match_titles", None)
-        self.state_machine.match_titles = self._renderer.md_env[
-            "match_titles"
-        ] = match_titles
+        """Perform a nested parse of the input block, with ``node`` as the parent.
 
+        :param block: The block of lines to parse.
+        :param input_offset: The offset of the first line of block,
+            to the starting line of the state (i.e. directive).
+        :param node: The parent node to attach the parsed content to.
+        :param match_titles: Whether to to allow the parsing of headings
+            (normally this is false,
+            since nested heading would break the document structure)
+        """
+        sm_match_titles = self.state_machine.match_titles
         with self._renderer.current_node_context(node):
             self._renderer.nested_render_text(
-                "\n".join(block), self._lineno + input_offset
+                "\n".join(block),
+                self._lineno + input_offset,
+                allow_headings=match_titles,
             )
         self.state_machine.match_titles = sm_match_titles
-        self._renderer.md_env["match_titles"] = render_match_titles
 
     def parse_target(self, block, block_text, lineno: int):
         """
@@ -164,35 +192,9 @@ class MockState:
     ) -> Tuple[List[nodes.Element], List[nodes.Element]]:
         """Parse text with only inline rules.
 
-        :return: (list of nodes, list of messages)
-
+        :returns: (list of nodes, list of messages)
         """
-        # TODO return messages?
-        messages: List[nodes.Element] = []
-        paragraph = nodes.paragraph("")
-
-        tokens = self._renderer.md.parseInline(text, self._renderer.md_env)
-        for token in tokens:
-            if token.map:
-                token.map = [token.map[0] + lineno, token.map[1] + lineno]
-
-        # here we instantiate a new renderer,
-        # so that the nested parse does not effect the current renderer,
-        # but we use the same env, so that link references, etc
-        # are added to the global parse.
-        from myst_parser.docutils_renderer import DocutilsRenderer
-
-        nested_renderer = DocutilsRenderer(self._renderer.md)
-        options = {k: v for k, v in self._renderer.config.items()}
-        options.update(
-            {
-                "document": self.document,
-                "current_node": paragraph,
-                "output_footnotes": False,
-            }
-        )
-        nested_renderer.render(tokens, options, self._renderer.md_env)
-        return paragraph.children, messages
+        return self.inliner.parse(text, lineno, self.memo, self._renderer.current_node)
 
     # U+2014 is an em-dash:
     attribution_pattern = re.compile("^((?:---?(?!-)|\u2014) *)(.+)")
@@ -357,12 +359,15 @@ class MockIncludeDirective:
             # i.e. relative to source directory
             try:
                 sphinx_env = self.document.settings.env
-                _, include_arg = sphinx_env.relfn2path(self.arguments[0])
-                sphinx_env.note_included(include_arg)
             except AttributeError:
                 pass
+            else:
+                _, include_arg = sphinx_env.relfn2path(self.arguments[0])
+                sphinx_env.note_included(include_arg)
             path = Path(include_arg)
         path = source_dir.joinpath(path)
+        # this ensures that the parent file is rebuilt if the included file changes
+        self.document.settings.record_dependencies.add(str(path))
 
         # read file
         encoding = self.options.get("encoding", self.document.settings.input_encoding)
@@ -453,21 +458,23 @@ class MockIncludeDirective:
             self.renderer.reporter.source = str(path)
             self.renderer.reporter.get_source_and_line = lambda l: (str(path), l)
             if "relative-images" in self.options:
-                self.renderer.config["relative-images"] = os.path.relpath(
+                self.renderer.md_env["relative-images"] = os.path.relpath(
                     path.parent, source_dir
                 )
             if "relative-docs" in self.options:
-                self.renderer.config["relative-docs"] = (
+                self.renderer.md_env["relative-docs"] = (
                     self.options["relative-docs"],
                     source_dir,
                     path.parent,
                 )
-            self.renderer.nested_render_text(file_content, startline + 1)
+            self.renderer.nested_render_text(
+                file_content, startline + 1, allow_headings=True
+            )
         finally:
             self.renderer.document["source"] = source
             self.renderer.reporter.source = rsource
-            self.renderer.config.pop("relative-images", None)
-            self.renderer.config.pop("relative-docs", None)
+            self.renderer.md_env.pop("relative-images", None)
+            self.renderer.md_env.pop("relative-docs", None)
             if line_func is not None:
                 self.renderer.reporter.get_source_and_line = line_func
             else:
